@@ -108,8 +108,9 @@ class API(object):
             return 5381  # 不知道这个数字有木有特殊含义
 
         # 不同客户端cookies返回的字段类型各有不同, 这里做一个折衷
-        string = cookies.get('qqmusic_key') or cookies['p_skey'] or \
-            cookies['skey'] or cookies['p_lskey'] or cookies['lskey']
+        string = cookies.get('qqmusic_key') or cookies.get('p_skey') or \
+            cookies.get('skey') or cookies.get('p_lskey') or \
+            cookies.get('lskey') or ''
         return djb2(string)
 
     def get_cover(self, mid, type_):
@@ -205,34 +206,88 @@ class API(object):
                 }
             }
         }
-        payload['comm'] = self.get_common_params()
         js = self.rpc(payload)
         data_songs = js['simsongs']['data']['songInfoList']
         return data_songs
 
-    def artist_detail(self, artist_id, page=1, page_size=50):
-        """获取歌手详情"""
-        path = '/v8/fcg-bin/fcg_v8_singer_track_cp.fcg'
-        url = api_base_url + path
-        params = {
-            'singerid': artist_id,
-            'order': 'listen',
-            'begin': page - 1,
-            'num': page_size,
-
-            # 有 newsong 字段时，服务端会返回含有 file 字段的字典
-            'newsong': 1
+    def artist_detail(self, artist_mid):
+        url = 'https://u.y.qq.com/cgi-bin/musics.fcg'
+        data = {
+            'req_0': {
+                'module': 'music.musichallSinger.SingerInfoInter',
+                'method': 'GetSingerDetail',
+                'param': {
+                    'singer_mids': [artist_mid],
+                    'pic': 1,
+                    'group_singer': 1,
+                    'wiki_singer': 1,
+                    'ex_singer': 1
+                }},
+            'comm': {
+                'g_tk': self.get_token_from_cookies(),
+                'uin': self._uin,
+                'format': 'json',
+            }
         }
-        resp = requests.get(url, params=params, timeout=self._timeout)
-        rv = resp.json()
-        return rv['data']
+        data_str = json.dumps(data)
+
+        params = {
+            '_': int(round(time.time() * 1000)),
+            'sign': _get_sign(data_str),
+            'data': data_str
+        }
+
+        resp = requests.get(url, params=params, headers=self._headers,
+                            cookies=self._cookies, timeout=self._timeout)
+        js = resp.json()
+
+        data = js['req_0']['data']['singer_list'][0]
+        data = {
+            'singer_id': data['basic_info']['singer_id'],
+            'singer_mid': data['basic_info']['singer_mid'],
+            'singer_name': data['basic_info']['name'],
+            'SingerDesc': data['ex_info']['desc'],
+        }
+        return data
+
+    def artist_songs(self, artist_id, page=1, page_size=50):
+        url = 'https://u.y.qq.com/cgi-bin/musics.fcg'
+        data = {
+            'req_0': {
+                'module': 'music.musichallSong.SongListInter',
+                'method': 'GetSingerSongList',
+                'param': {
+                    'singerid': artist_id,
+                    'begin': (page - 1) * page_size,
+                    'num': page_size,
+                    # 有 newsong 字段时，服务端会返回含有 file 字段的字典
+                    'newsong': 1
+                }},
+            'comm': {
+                'g_tk': self.get_token_from_cookies(),
+                'uin': self._uin,
+                'format': 'json',
+            }
+        }
+        data_str = json.dumps(data)
+
+        params = {
+            '_': int(round(time.time() * 1000)),
+            'sign': _get_sign(data_str),
+            'data': data_str
+        }
+
+        resp = requests.get(url, params=params, headers=self._headers,
+                            cookies=self._cookies, timeout=self._timeout)
+        js = resp.json()
+        return js['req_0']['data']
 
     def artist_albums(self, artist_id, page=1, page_size=20):
         url = api_base_url + '/v8/fcg-bin/fcg_v8_singer_album.fcg'
         params = {
             'singerid': artist_id,
             'order': 'time',
-            'begin': page - 1,  # TODO: 这里应该代表偏移量
+            'begin': (page - 1) * page_size,  # TODO: 这里应该代表偏移量
             'num': page_size
         }
         response = requests.get(url, params=params)
@@ -355,18 +410,56 @@ class API(object):
         return js['data']['cdlist']
 
     def get_recommend_songs_pid(self):
-        """get the playlist id of recommended songs"""
-        url = 'https://c.y.qq.com/node/musicmac/v6/index.html'
-        resp = requests.get(url, headers=self._headers,
-                            cookies=self._cookies, timeout=self._timeout)
-        # find this line, and the data-rid field value is the playlist id
-        # <a data-type="10014" data-rid="5187073319">今日私享</a>
-        text = resp.text
-        p = re.compile(r'data-rid="(\d+)">今日私享<')
-        m = p.search(text)
-        if m is None:
-            return None
-        return m.group(1)
+        data = {
+            'req_0': {
+                'module': 'recommend.RecommendFeedServer',
+                'method': 'get_recommend_feed',
+                'param': {
+                    'direction': 0,
+                    'page': 1,
+                    'v_cache': [],
+                    'v_uniq': [],
+                    's_num': 0
+                }
+            },
+        }
+        js = self.rpc(data)
+        disstid = js['req_0']['data']['v_shelf'][0]['v_niche'][0]['v_card'][1]['id']
+        return disstid
+
+    def recommend_playlists(self):
+        data = {
+            'recomPlaylist': {
+                'module': "playlist.HotRecommendServer",
+                'method': "get_hot_recommend",
+                'param': {
+                    'cmd': 2,
+                    'async': 1
+                }
+            },
+        }
+        js = self.rpc(data)
+        playlist = js['recomPlaylist']
+        return playlist['data']['v_hot']
+
+    def get_recommend_playlists_ids(self, index=0):
+        data = {
+            'req_0': {
+                'module': 'recommend.RecommendFeedServer',
+                'method': 'get_recommend_feed',
+                'param': {
+                    'direction': 0,
+                    'page': 1,
+                    'v_cache': [],
+                    'v_uniq': [],
+                    's_num': 0
+                }
+            },
+        }
+        js = self.rpc(data)
+        ids = [card['id']
+               for card in js['req_0']['data']['v_shelf'][index]['v_niche'][0]['v_card']]
+        return ids
 
     def get_lyric_by_songmid(self, songmid):
         url = api_base_url + '/lyric/fcgi-bin/fcg_query_lyric_new.fcg'
@@ -383,9 +476,17 @@ class API(object):
         return base64.b64decode(lyric).decode()
 
     def rpc(self, payload):
+        if 'comm' not in payload:
+            payload['comm'] = self.get_common_params()
         data_str = json.dumps(payload)
-        url = 'http://u.y.qq.com/cgi-bin/musicu.fcg?data=' + data_str
-        resp = requests.get(url, headers=self._headers, timeout=self._timeout)
+        params = {
+            '_': int(round(time.time() * 1000)),
+            'sign': _get_sign(data_str),
+            'data': data_str,
+        }
+        url = 'http://u.y.qq.com/cgi-bin/musicu.fcg'
+        resp = requests.get(url, params=params, headers=self._headers,
+                            timeout=self._timeout)
         js = resp.json()
         CodeShouldBe0.check(js)
         return js
