@@ -1,8 +1,8 @@
 import logging
 
-from fuocore.media import Quality, Media
-from fuocore.models import cached_field
-from fuocore.models import (
+from feeluown.media import Quality, Media
+from feeluown.models import (
+    cached_field,
     BaseModel,
     SongModel,
     LyricModel,
@@ -15,8 +15,7 @@ from fuocore.models import (
     ModelStage,
     SearchType,
 )
-
-from fuocore.reader import SequentialReader, wrap as reader_wrap
+from feeluown.utils.reader import SequentialReader, wrap as reader_wrap
 
 from .provider import provider
 
@@ -36,7 +35,7 @@ def _deserialize(data, schema_cls, gotten=True):
 
 def create_g(func, identifier, schema):
     data = func(identifier, page=1)
-    total = int(data['total'])
+    total = int(data['totalNum'] if schema == _ArtistSongSchema else data['total'])
 
     def g():
         nonlocal data
@@ -44,8 +43,8 @@ def create_g(func, identifier, schema):
             yield from ()
         else:
             page = 1
-            while data['list']:
-                obj_data_list = data['list']
+            while data['songList'] if schema == _ArtistSongSchema else data['list']:
+                obj_data_list = data['songList'] if schema == _ArtistSongSchema else data['list']
                 for obj_data in obj_data_list:
                     obj = _deserialize(obj_data, schema, gotten=False)
                     # FIXME: 由于 feeluown 展示歌手的 album 列表时，
@@ -226,13 +225,14 @@ class QQArtistModel(ArtistModel, QQBaseModel):
 
     @classmethod
     def get(cls, identifier):
-        data_artist = cls._api.artist_detail(identifier)
+        data_mid = cls._api.artist_songs(identifier, 1, 0)['singerMid']
+        data_artist = cls._api.artist_detail(data_mid)
         artist = _deserialize(data_artist, QQArtistSchema)
         artist.cover = cls._api.get_cover(artist.mid, 1)
         return artist
 
     def create_songs_g(self):
-        return create_g(self._api.artist_detail,
+        return create_g(self._api.artist_songs,
                         self.identifier,
                         _ArtistSongSchema)
 
@@ -301,6 +301,17 @@ class QQUserModel(UserModel, QQBaseModel):
         playlist = QQPlaylistModel.get(pid)
         return playlist.songs
 
+    @cached_field(ttl=5)  # ttl should be 0
+    def rec_playlists(self):
+        pids = self._api.get_recommend_playlists_ids()
+        # rec_playlists = [QQPlaylistModel.get(pid) for pid in pids]
+        playlists = self._api.recommend_playlists()
+        for pl in playlists:
+            pl['dissid'] = pl['content_id']
+            pl['dissname'] = pl['title']
+            pl['logo'] = pl['cover']
+        return [_deserialize(playlist, QQPlaylistSchema, False) for playlist in playlists]
+
 
 def search(keyword, **kwargs):
     type_ = SearchType.parse(kwargs['type_'])
